/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org Code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*

  The Sidebar API for 3rd parties

*/

#include "nsISupports.idl"

[scriptable, uuid(41a22fb6-39e2-45e1-95c5-7e8cea36575d)]
interface nsISidebar : nsISupports
{
    void addPanel(in wstring aTitle, in string aContentURL,
                  in string aCustomizeURL);
    void addPersistentPanel(in wstring aTitle, in string aContentURL,
                            in string aCustomizeURL);
    void addSearchEngine(in string engineURL, in string iconURL,
                         in wstring suggestedTitle, in wstring suggestedCategory);
};

[scriptable, uuid(4350fb73-9305-41df-a669-11d26222d420)]
interface nsISidebarExternal : nsISupports
{
    void AddSearchProvider(in string aDescriptionURL);
		unsigned long IsSearchProviderInstalled(in string aSearchURL);
};

%{ C++
// {577CB744-8CAF-11d3-AAEF-00805F8A4905} 
#define NS_SIDEBAR_CID \
{ 0x577cb744, 0x8caf, 0x11d3, { 0xaa, 0xef, 0x0, 0x80, 0x5f, 0x8a, 0x49, 0x5 } }

#define NS_SIDEBAR_CONTRACTID "@mozilla.org/sidebar;1"
%}

/*
 var isidebar = Components.interfaces.nsISidebar;
 var csidebar = Components.classes['@mozilla.org/sidebar;1'];
 var sidebar = csidebar.createInstance(isidebar);
*/
