/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Doug Turner <dougt@dougt.org>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"

interface nsIDOMWindow;

[scriptable, uuid(0ec7ed95-dc9e-4d20-a5e2-8fc6a03bce67)]
interface nsIAcceleration : nsISupports
{
  readonly attribute double alpha;
  readonly attribute double beta;
  readonly attribute double gamma;
};

[scriptable, uuid(3386BED8-7393-4704-8FFC-1EB2C35432FF)]
interface nsIAccelerationListener : nsISupports
{
  void onAccelerationChange(in nsIAcceleration aAcceleration);
};

[scriptable, uuid(4B04E228-0B33-43FC-971F-AF60CEDB1C21)]
interface nsIAccelerometer : nsISupports
{
  void addListener(in nsIAccelerationListener aListener);
  void removeListener(in nsIAccelerationListener aListener);

  void addWindowListener(in nsIDOMWindow aWindow);
  void removeWindowListener(in nsIDOMWindow aWindow);

};

/* for use by IPC system to notify non-chrome processes of 
 * accelerometer events
 */
[uuid(22dd1d8a-51bf-406f-8b6d-d1919f8f1c7d)]
interface nsIAccelerometerUpdate : nsIAccelerometer
{
  /* must be called on the main thread or else */
  void accelerationChanged(in double alpha, in double beta, in double gamma);
};
