/* ***** BEGIN LICENSE BLOCK *****
 Version: MPL 1.1/GPL 2.0/LGPL 2.1

 The contents of this file are subject to the Mozilla Public License Version
 1.1 (the "License"); you may not use this file except in compliance with
 the License. You may obtain a copy of the License at
 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 for the specific language governing rights and limitations under the
 License.

 The Original Code is Mozilla Communicator client code, released
 March 31, 1998.

 The Initial Developer of the Original Code is
 Blake Ross.
 Portions created by the Initial Developer are Copyright (C) 2002
 the Initial Developer. All Rights Reserved.

 Contributor(s):
   Kevin Gerich (webmail@kmgerich.com)

 Alternatively, the contents of this file may be used under the terms of
 either the GNU General Public License Version 2 or later (the "GPL"), or
 the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above. If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the GPL or the LGPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.

 ***** END LICENSE BLOCK ***** */

#palette-box {
  margin-top: 2px;
}
#palette-box > hbox > toolbarpaletteitem {
  padding: 2px;
  margin: 0px;
}

#main-box {
  padding: 12px;
}

#main-box > separator {
  -moz-appearance: none;
  border-bottom: none;
} 

#instructions {
  font: menu;
  font-weight: bold;
  line-height: 16pt;
}

hbox button {
  font: menu;
}

#main-box > box > button {
  min-height: 19px; /* aqua size for small buttons */
  font: message-box;
}
